* using placebo law interventions and the 50 states,
* this do-file creates a null distribution of the mean of 29 percentile ranks
* using the exact timing and overlap of
* states for the 29 actual treatments in our sample

* the distribution is formed by iterating the following process:
* 1. permute state numbers of placebo law synthetic control results, which already correspond to actual time distribution of 29 treatments
* 2. calculate "elasticities"
* 2. filter these shuffled results through the 29 actual treatments/donors, so that we exactly match treatment-donor overlap structure
* 3. calculate percentile ranks within each treatment event
* 4. calculate the mean of the "treated" state rank across the 29 treatment events

foreach configuration of numlist 1 {

	* use placebo results to create a list of state fips numbers, useful for shuffling later
	use if group == 1 using ${home}nonmwplacebo/data/compresults/synth3erateresults_pre99_post99${allopt}_nonmwplacebo.dta, clear
	egen tag = tag(statefipsnum)
	keep if tag == 1
	keep statefipsnum
	gen n = _n
	tempfile statefipsnumlist
	save `statefipsnumlist'

	* use placebo results to calculate individual treatmentXstate elasticities
	use if group == 1 using ${home}nonmwplacebo/data/compresults/synth3erateresults_pre99_post99${allopt}_nonmwplacebo.dta, clear
	keep if post == 1
	egen meandiff = mean(y_treated - y_synthetic), by(treatmentid statefipsnum)
	egen meansynth = mean(y_synthetic), by(treatmentid statefipsnum)
	gen postelast = (meandiff/meansynth)
	egen tag = tag(treatmentid statefipsnum)
	keep if tag == 1
	keep treatmentid statefipsnum postelast
	tempfile placeboresults
	save `placeboresults'
	
	* use actual treatments to create a template of actually treated/donor structure by event
	use ${cleandata}finaltreatmentsandcontrols_prelength99_postlength99.dta, clear
	egen _treatdate = min(date) if post == 1, by(treatmentid)
	egen treatdate = max(_treatdate), by(treatmentid)
	format %tq treatdate

	* define subsample of events used
	if `configuration' == 1 {
		local numevents = 29
	}
	* assert event count is correct
	tab treatmentid, nofreq
	assert r(r) == `numevents'

	* keep a record of treatment info: treatment date, donor count, window configuration, treated/donor status
	egen tag = tag(treatmentid statefipsnum)
	keep if tag == 1
	egen statecount = count(statefipsnum), by(treatmentid)
	keep treatmentid statefipsnum treated control treatdate prewindow postwindow statecount
	tempfile treatment_template
	save `treatment_template'

	* iterate the calculation of mean ranks
	tempfile iterationresults`configuration'
	tempname sim
	postfile `sim' iteration meanpctrank using `iterationresults`configuration''
	local maxiterations = $sigsamplesize
	set seed 37701244
	qui forvalues iteration=1/`maxiterations' {
		noi di "Working on configuration `configuration', iteration `iteration' of `maxiterations'"
		clear
		* shuffle statefipsnum
		* first create an old - new statefipsnum concordance
		use statefipsnum using `statefipsnumlist', clear
		gen uniform = runiform()
		sort uniform
		drop uniform
		gen n = _n
		rename statefipsnum oldstatefipsnum
		merge 1:1 n using `statefipsnumlist', assert(3) nogenerate
		drop n
		rename statefipsnum newstatefipsnum

		* then attach the new statefipsnum to placebo-law results
		rename oldstatefipsnum statefipsnum
		merge 1:m statefipsnum using `placeboresults', nogenerate assert(3)
		rename statefipsnum oldstatefipsnum
		
		* filter placebo-law results with new state numbers through treatment dataset
		* so that the placebo results match the exact overlap structure of actual treatments/donors
		* (who is treated and who are donors will be based on newstatefipsnum)
		rename newstatefipsnum statefipsnum
		merge 1:1 treatmentid statefipsnum using `treatment_template', keep(3) nogenerate

		* calculate percentile ranks of "elasticities" using Weibull rule
		egen postelastrank = rank(postelast), by(treatmentid)
		gen postelastprank = postelastrank/(statecount + 1)

		* calculate aggregated ranks of "treated"
		keep if treated == 1
		assert _N == `numevents'
		sum postelastprank
		local meanpctrank = r(mean)

		post `sim' (`iteration') (`meanpctrank')

	}
	postclose `sim'
	use `iterationresults`configuration'', clear
	gen byte numevents = `numevents'
	gen byte configuration = `configuration'
	save `iterationresults`configuration'', replace

} // configuration

use `iterationresults1', clear
save ${cleandata}meanpctrank_placebodistribution.dta, replace



* compare the iid and overlap-based mean rank distributions
qui foreach configuration of numlist 1 {

	if `configuration' == 1 {
		local numevents = 29
	}

	* loead iid-simulation critical values
	use if numevents == `numevents' using ${tabledata}pranksigcutoffs, clear
	foreach level in 0_5 2_5 5 95 97_5 99_5 {
		local uniformcut`level': di cut`level'
	}

	* load overlap-simulation mean ranks
	use if configuration == `configuration' using ${cleandata}meanpctrank_placebodistribution.dta, clear
	assert numevents == `numevents'
	sum iteration
	assert _N == r(max)

	* calculate critical values of the overlap-simulation mean ranks
	_pctile meanpctrank, percentiles(0.5 2.5 5.0 95.0 97.5 99.5)
	gen cut0_5 = r(r1)
	gen cut2_5 = r(r2)
	gen cut5 = r(r3)
	gen cut95 = r(r4)
	gen cut97_5 = r(r5)
	gen cut99_5 = r(r6)

	* calculate rejection rates using iid critical values but with overlaps-simulation data
	sum meanpctrank if meanpctrank < `uniformcut0_5' | meanpctrank > `uniformcut99_5'
	gen rejrate1_0 = r(N) / _N
	sum meanpctrank if meanpctrank < `uniformcut2_5' | meanpctrank > `uniformcut97_5'
	gen rejrate5_0 = r(N) / _N
	sum meanpctrank if meanpctrank < `uniformcut5' | meanpctrank > `uniformcut95'
	gen rejrate10_0 = r(N) / _N

	gen iterations = _N

	keep in 1
	keep *cut* rejrate* iterations
	noi di _n(2) "Simulated rejection rates for `numevents' events, configuration `configuration'"
	noi list rejrate* iterations, ab(30)

	save ${tabledata}rank_dependence_cutoffs`configuration'.dta, replace

}


